@@ -5,6 +5,8 @@ import inspect
 import re
 import hashlib
 import requests
+import os
+import mimetypes
 
 
 class BaseValidation:
@@ -83,18 +85,18 @@ class required(BaseValidation):
     def passes(self, attribute, key, dictionary):
         """The passing criteria for this rule.
 
-        This should return a True boolean value.
+        The key must exist in the dictionary and return a True boolean value.
+        The key can use * notation.
 
         Arguments:
             attribute {mixed} -- The value found within the dictionary
             key {string} -- The key in the dictionary being searched for.
-                            This key may or may not exist in the dictionary.
             dictionary {dict} -- The dictionary being searched
 
         Returns:
             bool
         """
-        return attribute or key in dictionary
+        return self.find(key, dictionary) and attribute
 
     def message(self, key):
         """A message to show when this rule fails
@@ -294,7 +296,7 @@ class is_future(BaseValidation):
 class email(BaseValidation):
     def passes(self, attribute, key, dictionary):
         return re.compile(
-            r"^[^.].+@([?)[a-zA-Z0-9-.])+.([a-zA-Z]{2,3}|[0-9]{1,3})(]?)$"
+            r"^[^.][^@]*@([?)[a-zA-Z0-9-.])+.([a-zA-Z]{2,3}|[0-9]{1,3})(]?)$"
         ).match(attribute)
 
     def message(self, attribute):
@@ -452,6 +454,21 @@ class in_range(BaseValidation):
         self.max = max
 
     def passes(self, attribute, key, dictionary):
+
+        attribute = str(attribute)
+
+        if attribute.isalpha():
+            return False
+
+        if "." in attribute:
+            try:
+                attribute = float(attribute)
+            except Exception:
+                pass
+
+        elif attribute.isdigit():
+            attribute = int(attribute)
+
         return attribute >= self.min and attribute <= self.max
 
     def message(self, attribute):
@@ -565,7 +582,7 @@ class strong(BaseValidation):
     def passes(self, attribute, key, dictionary):
         all_clear = True
 
-        if len(attribute) <= self.length:
+        if len(attribute) < self.length:
             all_clear = False
             self.length_check = False
 
@@ -791,6 +808,373 @@ class regex(BaseValidation):
         return "The {} matches pattern {} .".format(attribute, self.pattern)
 
 
+def parse_size(size):
+    """Parse humanized size into bytes"""
+    from hfilesize import FileSize
+
+    return FileSize(size, case_sensitive=False)
+
+
+class BaseFileValidation(BaseValidation):
+    def __init__(self, validations, messages={}, raises={}):
+        super().__init__(validations, messages=messages, raises=raises)
+        self.file_check = True
+        self.size_check = True
+        self.mimes_check = True
+        self.all_clear = True
+
+    def passes(self, attribute, key, dictionary):
+        if not os.path.isfile(attribute):
+            self.file_check = False
+            return False
+        if self.size:
+            file_size = os.path.getsize(attribute)
+            if file_size > self.size:
+                self.size_check = False
+                self.all_clear = False
+        if self.allowed_extensions:
+            mimetype, encoding = mimetypes.guess_type(attribute)
+            if mimetype not in self.allowed_mimetypes:
+                self.mimes_check = False
+                self.all_clear = False
+        return self.all_clear
+
+
+class file(BaseFileValidation):
+    def __init__(self, validations, size=False, mimes=False, messages={}, raises={}):
+        super().__init__(validations, messages=messages, raises=raises)
+        self.size = parse_size(size)
+
+        # parse allowed extensions to a list of mime types
+        self.allowed_extensions = mimes
+        if mimes:
+            self.allowed_mimetypes = list(
+                map(lambda mt: mimetypes.types_map.get("." + mt, None), mimes)
+            )
+
+    def message(self, attribute):
+        messages = []
+        if not self.file_check:
+            messages.append("The {} is not a valid file.".format(attribute))
+
+        if not self.size_check:
+            from hfilesize import FileSize
+
+            messages.append(
+                "The {} file size exceeds {:.02fH}.".format(
+                    attribute, FileSize(self.size)
+                )
+            )
+        if not self.mimes_check:
+            messages.append(
+                "The {} mime type is not valid. Allowed formats are {}.".format(
+                    attribute, ",".join(self.allowed_extensions)
+                )
+            )
+
+        return messages
+
+    def negated_message(self, attribute):
+        messages = []
+        if self.file_check:
+            messages.append("The {} is a valid file.".format(attribute))
+        if self.size_check:
+            from hfilesize import FileSize
+
+            messages.append(
+                "The {} file size is less or equal than {:.02fH}.".format(
+                    attribute, FileSize(self.size)
+                )
+            )
+        if self.mimes_check:
+            messages.append(
+                "The {} mime type is in {}.".format(
+                    attribute, ",".join(self.allowed_extensions)
+                )
+            )
+        return messages
+
+
+class image(BaseFileValidation):
+    def __init__(self, validations, size=False, messages={}, raises={}):
+        super().__init__(validations, messages=messages, raises=raises)
+        self.size = parse_size(size)
+        image_mimetypes = {
+            ext: mimetype
+            for ext, mimetype in mimetypes.types_map.items()
+            if mimetype.startswith("image")
+        }
+        self.allowed_extensions = list(image_mimetypes.keys())
+        self.allowed_mimetypes = list(image_mimetypes.values())
+
+    def message(self, attribute):
+        messages = []
+        if not self.file_check:
+            messages.append("The {} is not a valid file.".format(attribute))
+
+        if not self.size_check:
+            from hfilesize import FileSize
+
+            messages.append(
+                "The {} file size exceeds {:.02fH}.".format(
+                    attribute, FileSize(self.size)
+                )
+            )
+
+        if not self.mimes_check:
+            messages.append(
+                "The {} file is not a valid image. Allowed formats are {}.".format(
+                    attribute, ",".join(self.allowed_extensions)
+                )
+            )
+
+        return messages
+
+    def negated_message(self, attribute):
+        messages = []
+        if self.file_check:
+            messages.append("The {} is a valid file.".format(attribute))
+        if self.size_check:
+            from hfilesize import FileSize
+
+            messages.append(
+                "The {} file size is less or equal than {:.02fH}.".format(
+                    attribute, FileSize(self.size)
+                )
+            )
+
+        if self.mimes_check:
+            messages.append("The {} file is a valid image.".format(attribute))
+
+        return messages
+
+
+class video(BaseFileValidation):
+    def __init__(self, validations, size=False, messages={}, raises={}):
+        super().__init__(validations, messages=messages, raises=raises)
+        self.size = parse_size(size)
+
+        video_mimetypes = {
+            ext: mimetype
+            for ext, mimetype in mimetypes.types_map.items()
+            if mimetype.startswith("video")
+        }
+
+        self.allowed_extensions = list(video_mimetypes.keys())
+        self.allowed_mimetypes = list(video_mimetypes.values())
+
+    def message(self, attribute):
+        messages = []
+        if not self.file_check:
+            messages.append("The {} is not a valid file.".format(attribute))
+
+        if not self.size_check:
+            from hfilesize import FileSize
+
+            messages.append(
+                "The {} file size exceeds {:.02fH}.".format(
+                    attribute, FileSize(self.size)
+                )
+            )
+
+        if not self.mimes_check:
+            messages.append(
+                "The {} file is not a valid video. Allowed formats are {}.".format(
+                    attribute, ",".join(self.allowed_extensions)
+                )
+            )
+
+        return messages
+
+    def negated_message(self, attribute):
+        messages = []
+        if self.file_check:
+            messages.append("The {} is a valid file.".format(attribute))
+
+        if self.size_check:
+            from hfilesize import FileSize
+
+            messages.append(
+                "The {} file size is less or equal than {:.02fH}.".format(
+                    attribute, FileSize(self.size)
+                )
+            )
+
+        if self.mimes_check:
+            messages.append("The {} file is a valid video.".format(attribute))
+
+        return messages
+
+
+class postal_code(BaseValidation):
+    def __init__(self, validations, locale, messages={}, raises={}):
+        super().__init__(validations, messages=messages, raises=raises)
+        from .resources.postal_codes import PATTERNS
+
+        self.locales = []
+        self.patterns = []
+        self.patterns_example = []
+        self.locales = locale.split(",")
+
+        for locale in self.locales:
+            pattern_dict = PATTERNS.get(locale, None)
+            if pattern_dict is None or pattern_dict["pattern"] is None:
+                raise NotImplementedError(
+                    "Unsupported country code {}. Check that it is a ISO 3166-1 country code or open a PR to require support of this country code.".format(
+                        locale
+                    )
+                )
+            else:
+                self.patterns.append(pattern_dict["pattern"])
+                self.patterns_example.append(pattern_dict["example"])
+
+    def passes(self, attribute, key, dictionary):
+        for pattern in self.patterns:
+            # check that at least one pattern match attribute
+            if re.compile(r"{}".format(pattern)).match(attribute):
+                return True
+        return False
+
+    def message(self, attribute):
+        return "The {} is not a valid {} postal code. Valid {} {}.".format(
+            attribute,
+            ",".join(self.locales),
+            "examples are" if len(self.locales) > 1 else "example is",
+            ",".join(self.patterns_example),
+        )
+
+    def negated_message(self, attribute):
+        return "The {} is a valid {} postal code.".format(attribute, self.locale)
+
+
+class different(BaseValidation):
+    """The field under validation must be different than an other given field."""
+
+    def __init__(self, validations, other_field, messages={}, raises={}):
+        super().__init__(validations, messages=messages, raises=raises)
+        self.other_field = other_field
+
+    def passes(self, attribute, key, dictionary):
+        other_value = dictionary.get(self.other_field, None)
+        return attribute != other_value
+
+    def message(self, attribute):
+        return "The {} value must be different than {} value.".format(
+            attribute, self.other_field
+        )
+
+    def negated_message(self, attribute):
+        return "The {} value be the same as {} value.".format(
+            attribute, self.other_field
+        )
+
+
+class uuid(BaseValidation):
+    """The field under validation must be a valid UUID. The UUID version standard
+    can be precised (1,3,4,5)."""
+
+    def __init__(self, validations, version=4, messages={}, raises={}):
+        super().__init__(validations, messages=messages, raises=raises)
+        self.version = version
+        self.uuid_type = "UUID"
+        if version:
+            self.uuid_type = "UUID {0}".format(self.version)
+
+    def passes(self, attribute, key, dictionary):
+        from uuid import UUID
+
+        try:
+            uuid_value = UUID(str(attribute))
+            return uuid_value.version == int(self.version)
+        except ValueError:
+            return False
+
+    def message(self, attribute):
+        return "The {} value must be a valid {}.".format(attribute, self.uuid_type)
+
+    def negated_message(self, attribute):
+        return "The {} value must not be a valid {}.".format(attribute, self.uuid_type)
+
+
+class required_if(BaseValidation):
+    """The field under validation must be present and not empty only
+    if an other field has a given value."""
+
+    def __init__(self, validations, other_field, value, messages={}, raises={}):
+        super().__init__(validations, messages=messages, raises=raises)
+        self.other_field = other_field
+        self.value = value
+
+    def passes(self, attribute, key, dictionary):
+        if dictionary.get(self.other_field, None) == self.value:
+            return required.passes(self, attribute, key, dictionary)
+
+        return True
+
+    def message(self, attribute):
+        return "The {} is required because {}={}.".format(
+            attribute, self.other_field, self.value
+        )
+
+    def negated_message(self, attribute):
+        return "The {} is not required because {}={} or {} is not present.".format(
+            attribute, self.other_field, self.value, self.other_field
+        )
+
+
+class required_with(BaseValidation):
+    """The field under validation must be present and not empty only
+    if any of the other specified fields are present."""
+
+    def __init__(self, validations, other_fields, messages={}, raises={}):
+        super().__init__(validations, messages=messages, raises=raises)
+        if not isinstance(other_fields, list):
+            if "," in other_fields:
+                self.other_fields = other_fields.split(",")
+            else:
+                self.other_fields = [other_fields]
+        else:
+            self.other_fields = other_fields
+
+    def passes(self, attribute, key, dictionary):
+        for field in self.other_fields:
+            if field in dictionary:
+                return required.passes(self, attribute, key, dictionary)
+        else:
+            return True
+
+    def message(self, attribute):
+        fields = ",".join(self.other_fields)
+        return "The {} is required because {} is present.".format(
+            attribute,
+            "one in {}".format(fields)
+            if len(self.other_fields) > 1
+            else self.other_fields[0],
+        )
+
+    def negated_message(self, attribute):
+        return "The {} is not required because {} {} is not present.".format(
+            attribute,
+            "none of" if len(self.other_fields) > 1 else "",
+            ",".join(self.other_fields),
+        )
+
+
+class distinct(BaseValidation):
+    """When working with list, the field under validation must not have any
+    duplicate values."""
+
+    def passes(self, attribute, key, dictionary):
+        # check if list contains duplicates
+        return len(set(attribute)) == len(attribute)
+
+    def message(self, attribute):
+        return "The {} field has duplicate values.".format(attribute)
+
+    def negated_message(self, attribute):
+        return "The {} field has only different values.".format(attribute)
+
+
 def flatten(iterable):
 
     flat_list = []
@@ -901,10 +1285,14 @@ class ValidationFactory:
             contains,
             date,
             does_not,
+            different,
+            distinct,
             equals,
             email,
             exists,
+            file,
             greater_than,
+            image,
             in_range,
             is_future,
             is_in,
@@ -920,12 +1308,17 @@ class ValidationFactory:
             numeric,
             one_of,
             phone,
+            postal_code,
             regex,
             required,
+            required_if,
+            required_with,
             string,
             strong,
             timezone,
             truthy,
+            uuid,
+            video,
             when,
         )
 
